=begin
sptdiary_style.rb: stratified paragraph tDiary style class for tDiary 2.x format.
This is a slightly modified version of tdiary_style.rb (Copyright TADA Tadashi)

==Usage
If you want to use this style, add @style into tdiary.conf below:
   @style = 'sptDiary'

And you should modify your CSS in 'theme/' directiory. 
This style add tags and classes below:
   "<h4>", "<h5>", "<h6>" in addition to "<h3>"
   "subsection","subsubsection","subsubsubsection" in addition to "subsection"

==Lisence
Copyright 2006 KASHIWAI,Isana (isana at ma.neweb.ne.jp)
This script based on tdiary_style.rb (Copyright TADA Tadashi)

You can redistribute and/or modify this script
under the terms of the GNU General Public License Version 2 (see gpl.txt)

==Alternations
(Sorry, here is modified methods list only.)

"SptdiarySection" class based on "TdiarySection" class in tdiary_style.rb.
Modified methods are listed below:
   initialize
   to_src
   categorized_subtitle

"SptdiaryDiary" class based on "TdiaryDiary" class in tdiary_style.rb.
Modified methods are listed below:
   style ("tDiary" to "sptdiary")
   append  ("TdiarySection" to "SptdiarySection")
   add_section ("TdiarySection" to "SptdiarySection")
   to_html4
   to_chtml

==Last modified: 
Wed, 8 Mar 2006 12:30:00 +0900

=end

module TDiary

	class SptdiarySection
		attr_reader :subtitle, :body, :author
		attr_reader :categories, :stripped_subtitle

		alias :subtitle_to_html :subtitle
		alias :stripped_subtitle_to_html :stripped_subtitle

		def initialize( fragment, author = nil )
			@author = author

			if fragment[0] == ?\* then
				@subtitle, @body = fragment.split( /\n/, 2 )
				/^(\*+)/ =~ @subtitle
				@h_mark = $1.to_s
				@subtitle.sub!( /^\*+\s*/, '' )
				if @subtitle == "" then
					@h_mark = @h_mark+"\n"
				end
			else
				@subtitle = nil
				@body = fragment.dup
			end

			@categories = get_categories
			@stripped_subtitle = strip_subtitle

		end
	
		def subtitle=(subtitle)
			cat_str = ""
			@categories.each {|cat|
				cat_str << "[#{cat}]"
			}
			cat_str << " " unless cat_str.empty?
			@subtitle = subtitle ? (cat_str + subtitle) : nil
			@stripped_subtitle = strip_subtitle
		end
	
		def body=(str)
			@body = str
		end
	
		def categories=(categories)
			@categories = categories
			cat_str = ""
			categories.each {|cat|
				cat_str << "[#{cat}]"
			}
			cat_str << " " unless cat_str.empty?
			@subtitle = @subtitle ? (cat_str + @stripped_subtitle) : nil
			@stripped_subtitle = strip_subtitle
		end
	
		def to_src
			s = "#{@h_mark}"
			if @stripped_subtitle then
				s += "[#{@author}]" if @author
				cat_str = ""
				@categories.each {|cat|
				  cat_str << "[#{cat}]"
				}
				cat_str << " " unless cat_str.empty?
				s += cat_str
				s += @stripped_subtitle + "\n"
			else
				s += '' unless @body=~/\A\s</
			end
			"#{s}#{@body}\n\n"
		end

		def body_to_html
			html = ""
			tag = false
			if @body
				@body.each do |p|
					if p[0] == ?< then
						html = @body.dup
						break
					end
					html << "<p>#{p}</p>"
				end
			end
			html
		end

		def to_s
			"subtitle=#{@subtitle}, body=#{@body}"
		end

		def categorized_subtitle
			"#{@h_mark}" + @categories.collect do |c|
				%Q|<%= category_anchor("#{c}") %>|
			end.join + ' ' + @stripped_subtitle.to_s
		end

	private
		def get_categories
			return [] unless @subtitle
			cat = /^(\[(.*?)\])+/.match(@subtitle).to_a[0]
			return [] unless cat
			cat.scan(/\[(.*?)\]/).collect do |c|
				c[0].split(/,/)
			end.flatten
		end

		def strip_subtitle
			return nil unless @subtitle
			r = @subtitle.sub(/^(\[(.*?)\])+\s*/,'')
			if r == ""
				nil
			else
				r
			end
		end
	end

	class SptdiaryDiary
		include DiaryBase
		include CategorizableDiary
	
		def initialize( date, title, body, modified = Time::now )
			init_diary
			replace( date, title, body )
			@last_modified = modified
		end
	
		def style
			'sptDiary'
		end
	
		def replace( date, title, body )
			set_date( date )
			set_title( title )
			@sections = []
			append( body )
		end
	
		def append( body, author = nil )
			body.gsub( /\r/, '' ).split( /\n\n+/ ).each do |fragment|
				section = SptdiarySection::new( fragment, author )
				@sections << section if section
			end
			@last_modified = Time::now
			self
		end
	
		def each_section
			@sections.each do |section|
				yield section
			end
		end
	
		def add_section(subtitle, body)
			sec = SptdiarySection::new("\n\n ")
			sec.subtitle = subtitle
			sec.body     = body
			@sections << sec
			@sections.size
		end
	
		def delete_section(index)
			@sections.delete_at(index - 1)
		end
	
		def to_src
			src = ''
			each_section do |section|
				src << section.to_src
			end
			src
		end
	
		def to_html( opt, mode = :HTML )
			case mode
			when :CHTML
				to_chtml( opt )
			else
				to_html4( opt )
			end
		end
	
		def to_html4( opt )
			idx = 1
			r = ''
			subtitle_str = ''
			each_section do |section|
				h = '' 
				if /^\*\*\*\*/ =~ section.categorized_subtitle then
					h = 'h6'
				elsif /^\*\*\*/ =~ section.categorized_subtitle then
					h = 'h5'
				elsif /^\*\*/ =~ section.categorized_subtitle then
					h = 'h4'
				elsif /^\*/ =~ section.categorized_subtitle then
					h = 'h3'
				end
				
				if h == 'h3' then
					r << %Q[<div class="section">\n]
				elsif h == 'h4' then
					r << %Q[<div class="subsection">\n]
				elsif h == 'h5' then
					r << %Q[<div class="subsubsection">\n]
				elsif h == 'h6' then
					r << %Q[<div class="subsubsubsection">\n]
				else
					r << %Q[<div class="section">\n]
				end

				if section.subtitle then
					subtitle_str=section.categorized_subtitle.sub( /^\*+/, '' ).strip
					if subtitle_str !=''
						if h == 'h5' || h == 'h6' then
							r << %Q[<#{h}>]
							idx -= 1
						else
						
						r << %Q[<#{h}><a ]
							if opt['anchor'] then
								r << %Q[name="p#{'%02d' % idx}" ]
							end
							r << %Q[href="#{opt['index']}<%=anchor "#{date.strftime( '%Y%m%d' )}#p#{'%02d' % idx}" %>">#{opt['section_anchor']}</a> ]
							if opt['multi_user'] and section.author then
								r << %Q|[#{section.author}]|
							end
						end

					r << %Q[#{subtitle_str}</#{h}>\n]
					idx += 1
					end
				end
				if section.body then
					if h == 'h3'||'h4' then
						r << %Q[<%= section_enter_proc( Time::at( #{date.to_i} ) ) %>\n]
					end
					if /\~$/  =~ section.body then
						body_str = section.body.gsub(/\~$\s/,'<br>')
					else
						body_str = section.body
					end
					if /^</ =~ body_str then
						r << %Q[#{body_str}]
					else
						r << %Q[<p>#{body_str.collect{|l|l.chomp.sub( /^[ ]/e, '')}.join( "</p>\n<p>" )}</p>\n]
					end
					if h == 'h3'||'h4' then
						r << %Q[<%= section_leave_proc( Time::at( #{date.to_i} ) ) %>\n]
					end
				end
				r << %Q[</div>\n]
			end
			r
		end
		
		def to_chtml( opt )
			idx = 0
			r = ''
			each_section do |section|
				if section.subtitle then
					r << %Q[<H3><A NAME="p#{'%02d' % idx += 1}">*</A> #{section.subtitle}</H3>]
				end
				if section.body then
					r << %Q[<%= section_enter_proc( Time::at( #{date.to_i} ) ) %>\n]
					if /\~$/  =~ section.body then
						body_str = section.body.gsub(/\~$\s/,'<BR>')
					else
						body_str = section.body
					end
					if /^</ =~ body_str then
						idx += 1
						r << body_str
					elsif section.subtitle
						r << %Q[<P>#{body_str.collect{|l|l.chomp.sub( /^[ ]/e, '' )}.join( "</P>\n<P>" )}</P>]
					else
						r << %Q[<P><A NAME="p#{'%02d' % idx += 1}">*</A> ]
						if opt['multi_user'] and section.author then
							r << %Q|[#{section.author}]|
						end
						r << %Q[#{body_str.collect{|l|l.chomp.sub( /^[ ]/e, '' )}.join( "</P>\n<P>" )}</P>]
					end
				r << %Q[<%= section_leave_proc( Time::at( #{date.to_i} ) ) %>\n]
				end
			end
			r
		end
	
		def to_s
			"date=#{date.strftime('%Y%m%d')}, title=#{title}, body=[#{@sections.join('][')}]"
		end
	end
end
